#include <cassert>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

#include <mpi.h>

#include "graph.h"

int main(int argc, char** argv)
{
   assert(argc > 1);
   MPI_Init(&argc, &argv);
   
   // how many processes are there?
   int size = 0;  
   MPI_Comm_size(MPI_COMM_WORLD, &size); 
   
   // what is the rank of this process?
   int rank = 0;
   MPI_Comm_rank(MPI_COMM_WORLD, &rank); 
   
   graph::Graph g;
   int message_size = 0;
   std::stringstream text;
   
   if(rank == 0)
   {
      std::ifstream indata(argv[1]);
      if(!indata)
      {
         std::cout << "Error! File " << argv[1] << " cannot be read.\n";
         return EXIT_FAILURE;
      }
      indata >> g;
      indata.close();
      std::cout << "\nContent of graph g, read from file by rank 0:\n" << g << "\n";
      
      std::cout << "\nRank 0 writing g into a string stream.\n";
      text << g; 
      std::cout.flush(); 
      
      message_size = text.str().size() + 1;  // leave one for terminal zero
      MPI_Send(&message_size, 1, MPI_INT, 1, 1, MPI_COMM_WORLD);
      
      MPI_Send(text.str().c_str(), message_size, MPI_CHAR, 1, 2, MPI_COMM_WORLD);
   }
   
   if(rank == 1)
   {
      MPI_Recv(&message_size, 1, MPI_INT, 0, 1, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
      std::cout << "Rank 1 expecting to receive " << message_size << " characters.\n";
      std::cout.flush();
      
      char* buffer = new char[message_size]();
      MPI_Recv(buffer, message_size, MPI_CHAR, 0, 2, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
      std::cout << "Rank 1 has received the content.\n";
      std::cout.flush();
      
      text << buffer;
      delete[] buffer;
      buffer = nullptr;
      
      text >> g;
      std::cout << "\nContent of graph g, read from rank 0 by rank 1:\n" << g << "\n";
      std::cout.flush();
   }
   
   MPI_Finalize();
}
